//
//  MyScene.m
//  SpaceInvadersTutorial
//
//  Created by Barbara Köhler on 6/13/13.
//  Copyright (c) 2013 Barbara Köhler. All rights reserved.
//

#import "MyScene.h"
#import "Player.h"
#import "Constants.h"
#import "Enemy.h"
#import "GameOverScene.h"

#import "AudioFile.h"
#import "GCHelper.h"
#import "AppDelegate.h"

@import GameKit;
@import AVFoundation;

@interface MyScene() {
    Player* player;
    NSMutableArray* enemies;
    float enemySpawnInterval;
    int64_t score;
    SKLabelNode* scoreLabel;
    SKLabelNode* pauseLabel;
    NSDate *startTime;
    BOOL gamePaused;
    
    AudioFile* backgroundMusic;
    AudioFile* gameOverSound;
    
}

@end

@implementation MyScene

-(id)initWithSize:(CGSize)size {    
    if (self = [super initWithSize:size]) {
        /* Setup your scene here */
        
        player = [[Player alloc] init];
        player.position = CGPointMake(CGRectGetMidX(self.frame), 0 + player.frame.size.height/2);
        [self addChild:player];
        
        gamePaused = NO;
        
        [self setupEnemies];
        
        /* Setup the score label */
        scoreLabel = [SKLabelNode labelNodeWithFontNamed:@"Chalkduster"];
        scoreLabel.text = @"0";
        scoreLabel.fontSize = 42;
        scoreLabel.position = CGPointMake(CGRectGetMidX(self.frame), CGRectGetMaxY(self.frame) - 100.0);
        [self addChild:scoreLabel];
        
        /* Setup the pause label */
        pauseLabel = [SKLabelNode labelNodeWithFontNamed:@"Chalkduster"];
        pauseLabel.text = @"Game Paused. Touch anywhere to begin.";
        pauseLabel.fontSize = 12;
        pauseLabel.position = CGPointMake(CGRectGetMidX(self.frame), CGRectGetMinY(self.frame) + 100.0);
        
        startTime = [[NSDate alloc] init];
        backgroundMusic = [[AudioFile alloc] initWithFileNamed:kBackgroundMusic numberOfLoops:-1];
        [backgroundMusic play];
        
        gameOverSound = [[AudioFile alloc] initWithFileNamed:kGameOverSound];
        score = 0;
        
        /* Register with the delegate so that it can pause the game if necessary */
        ((AppDelegate*)[[UIApplication sharedApplication] delegate]).gameScene = self;
    }
    return self;
}

-(void) setupEnemies {
    enemySpawnInterval = 1.0f;
    
    // use a temporary enemy sprite to get the images size
    SKSpriteNode* temporaryEnemy = [SKSpriteNode spriteNodeWithImageNamed:kAlienTestSizeImage];
    float enemyWidth = temporaryEnemy.frame.size.width;
    
    // use as many enemies as can fit on the screen
    int numberOfEnemies = self.frame.size.width / enemyWidth;
    
    // Initialize the array containing the enemies
    enemies = [NSMutableArray arrayWithCapacity:numberOfEnemies];
    for (int i = 0; i < numberOfEnemies; i++) {
        Enemy* enemy = [[Enemy alloc] initWithParent:self];
        [enemies addObject:enemy];
    }
    [self resetEnemies];
}

-(void) resetEnemies {
    for (int i = 0; i < [enemies count]; i++) {
        [[enemies objectAtIndex:i] resetAtPosition:i];
    }
    [NSTimer scheduledTimerWithTimeInterval:enemySpawnInterval target:self selector:@selector(spawnEnemy:) userInfo:nil repeats:YES];
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {   
    /* Called when a touch begins */
    if (!gamePaused) {
        for (UITouch *touch in touches) {
            CGPoint location = [touch locationInNode:self];
            if (location.x < CGRectGetMidX(player.frame)) {
                player.velocity = CGPointMake(-1, 0);
            } else {
                player.velocity = CGPointMake(1, 0);
            }
        }
    }
    else {
        [self resumeGame];
    }
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    player.velocity = CGPointMake(0, 0);
}

-(void)update:(CFTimeInterval)currentTime {
    /* Called before each frame is rendered */
    [player update];
    if (!gamePaused) {
        score++;
    }
    scoreLabel.text = [NSString stringWithFormat:@"%lld",score];
    if (!gamePaused) {
        [self checkForCollision];
    }
}

-(void) spawnEnemy:(CFTimeInterval) delta {
    if ([enemies count] != 0) {
        // find enemy which is not currently moving
        for (int i = 0; i < 100; i++) {
            int randomEnemyIndex = arc4random() % [enemies count];
            Enemy* enemy = [enemies objectAtIndex:randomEnemyIndex];
            if (![enemy isActive]) {
                [enemy move];
                break;
            }
        }
    }
}

- (void)pauseGame
{
    gamePaused = YES;
    for (Enemy* enemy in enemies) {
        [enemy removeFromParent];
    }
    [enemies removeAllObjects];
    [self addChild:pauseLabel];

}

- (void)resumeGame
{
    gamePaused = NO;
    
    [pauseLabel removeFromParent];
    [self setupEnemies];
}

-(void) checkForCollision {
    for (Enemy* enemy in enemies) {
        if ([enemy collidesWithPlayerFrame:[player frame]]) {
            [self handleGameOver];
        }
    }
}

- (void)handleGameOver {
    [gameOverSound play];
    NSDate *endTime = [[NSDate alloc] init];
    
    SKScene* gameOverScene = [[GameOverScene alloc] initWithSize:self.size score:score];
    [[GCHelper defaultHelper] reportScore:score forLeaderboardID:kLeaderBoardIdentifier];
    
    // check if the score is enough to earn an achievement
    if (score > 500) {
        [[GCHelper defaultHelper] reportAchievementIdentifier:kAchievement500Score percentComplete:100.0];
        // check if the score is enough to earn another achievement
        if (score > 1000) {
            [[GCHelper defaultHelper] reportAchievementIdentifier:kAchievement1000Score percentComplete:100.0];
        }
    }

    // check if the time interval in seconds is greater than 30s
    if ([endTime timeIntervalSinceDate:startTime] > 30.0) {
        [[GCHelper defaultHelper] reportAchievementIdentifier:kAchievement30Sec percentComplete:100.0];
        // check if the time interval in seconds is greater than 1 min
        if ([endTime timeIntervalSinceDate:startTime] > 60.0) {
            [[GCHelper defaultHelper] reportAchievementIdentifier:kAchievement1Min percentComplete:100.0];

        }
    }
    ((AppDelegate*)[[UIApplication sharedApplication] delegate]).gameScene = nil;
    [self.view presentScene: gameOverScene];

}

@end
